paths method

List<List<T>> paths(
  1. T start,
  2. T target
)
inherited

Returns all paths from start to target including cycles.

  • Returns an empty list if no path was found.
  • To exclude cycles and list only the shortest paths use the method shortestPaths.

Implementation

List<List<T>> paths(T start, T target) => crawler.paths(start, target);